@@ -172,6 +172,17 @@ enum _X(chipset_type) {
     _X(NVIDIA_NFORCE2),
     _X(NVIDIA_NFORCE3),
     _X(NVIDIA_GENERIC),
+    _X(ATI_RS100),
+    _X(ATI_RS200),
+    _X(ATI_RS250),
+    _X(ATI_RS300_100),
+    _X(ATI_RS300_133),
+    _X(ATI_RS300_166),
+    _X(ATI_RS300_200),
+    _X(ATI_RS350_100),
+    _X(ATI_RS350_133),
+    _X(ATI_RS350_166),
+    _X(ATI_RS350_200),
 };
 
 typedef struct _X(_agp_version) {
