@@ -1,24 +1,17 @@
 /****************************************************************************
- *																			*
- *  Copyright (c) 1999-2000 FGL Graphics, a division of SONICblue Inc.   	*
- *  All Rights Reserved.													*
- *																			*
- *  This software and its associated documentation contains proprietary,	*
- *  confidential and trade secret information of FGL Graphics,				*
- *  a division of SONICblue Inc.											*
- *																			*
- *  a)  no part may be disclosed, distributed, reproduced, transmitted,		*
- *      transcribed, stored in a retrieval system, adapted or translated	*
- *      in any form or by any means electronic, mechanical, magnetic,		*
- *      optical, chemical, manual or otherwise,								*
- *																			*
- *  and																		*
- *																			*
- *  b)  the recipient is not entitled to discover through reverse			*
- *      engineering or reverse compiling or other such techniques or		*
- *      processes the trade secrets contained therein or in the				*
- *		documentation.														*
- *																			*
+ *                                                                          *
+ * Copyright 1999-2005 ATI Technologies Inc., Markham, Ontario, CANADA.     *
+ * All Rights Reserved.                                                     *
+ *                                                                          *
+ * Your use and or redistribution of this software in source and \ or       *
+ * binary form, with or without modification, is subject to: (i) your       *
+ * ongoing acceptance of and compliance with the terms and conditions of    *
+ * the ATI Technologies Inc. software End User License Agreement; and (ii)  *
+ * your inclusion of this notice in any version of this software that you   *
+ * use or redistribute.  A copy of the ATI Technologies Inc. software End   *
+ * User License Agreement is included with this software and is also        *
+ * available by contacting ATI Technologies Inc. at http://www.ati.com      *
+ *                                                                          *
  ****************************************************************************/
 
 #ifndef _FIREGL_PUBLIC_H_
@@ -37,6 +30,134 @@
 #define _agp_memory             agp_memory 
 #endif 
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
+
+#define REMAP_PAGE_RANGE_FN remap_pfn_range
+#define REMAP_PAGE_RANGE_STR "remap_pfn_range"
+#define REMAP_PAGE_RANGE_OFF(vma) ((vma)->vm_pgoff)
+
+#else /* LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,9) */
+
+#define REMAP_PAGE_RANGE_FN remap_page_range
+#define REMAP_PAGE_RANGE_STR "remap_page_range"
+#define REMAP_PAGE_RANGE_OFF(vma) __ke_vm_offset(vma)
+
+#endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9) */
+
+#define REMAP_PAGE_RANGE(vma) REMAP_PAGE_RANGE_FN(FGL_VMA_API_PASS \
+                                                  (vma)->vm_start,	\
+                                                  REMAP_PAGE_RANGE_OFF(vma), \
+                                                  (vma)->vm_end - (vma)->vm_start, \
+                                                  (vma)->vm_page_prot)
+
+
+/* Page table macros */
+
+#define PGD_OFFSET(mm, pgd_p, pte_linear)	\
+do { \
+    pgd_p = pgd_offset(mm, pte_linear); \
+} while(0)
+
+#define PGD_OFFSET_K(pgd_p, pte_linear)	\
+do { \
+    pgd_p = pgd_offset_k(pte_linear); \
+} while(0)
+
+#define PGD_PRESENT(pgd_p) \
+do { \
+    if (!pgd_present(*(pgd_p)))	\
+    { \
+        __KE_ERROR("FATAL ERROR: User queue buffer not present! (pgd)\n"); \
+        return (unsigned long)NOPAGE_SIGBUS;   /* Something bad happened; generate SIGBUS */ \
+        /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
+    } \
+} while(0)
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
+
+#define PMD_OFFSET(pmd_p, pgd_p, pte_linear) \
+do { \
+    pud_t *pud_p = pud_offset(pgd_p, pte_linear); \
+    if (!pud_present(*(pud_p)))	\
+    { \
+        __KE_ERROR("FATAL ERROR: User queue buffer not present! (pud)\n"); \
+        return (unsigned long)NOPAGE_SIGBUS;   /* Something bad happened; generate SIGBUS */ \
+        /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
+    } \
+    pmd_p = pmd_offset(pud_p, pte_linear); \
+} while(0)
+
+#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11) */
+
+#define PMD_OFFSET(pmd_p, pgd_p, pte_linear)	\
+do { \
+    pmd_p = pmd_offset(pgd_p, pte_linear); \
+} while(0)
+
+#endif
+
+#define PMD_PRESENT(pmd_p) \
+do { \
+    if (!pmd_present(*(pmd_p)))	\
+    { \
+        __KE_ERROR("FATAL ERROR: User queue buffer not present! (pmd)\n"); \
+        return (unsigned long)NOPAGE_SIGBUS;   /* Something bad happened; generate SIGBUS */ \
+        /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
+    } \
+} while(0)
+
+#ifdef pte_offset_atomic
+#define PTE_OFFSET(pte, pmd_p, pte_linear) \
+do { \
+    pte_t* pte_p; \
+    pte_p = pte_offset_atomic(pmd_p, pte_linear); \
+    pte = *pte_p; \
+    pte_kunmap(pte_p); \
+} while(0)
+#else
+#ifdef pte_offset_map
+#define PTE_OFFSET(pte, pmd_p, pte_linear) \
+do { \
+    pte_t* pte_p; \
+    pte_p = pte_offset_map(pmd_p, pte_linear); \
+    pte = *pte_p; \
+    pte_unmap(pte_p); \
+} while(0)
+#else
+#ifdef pte_offset_kernel
+#define PTE_OFFSET(pte, pmd_p, pte_linear) \
+do { \
+    pte_t* pte_p; \
+    pte_p = pte_offset_kernel(pmd_p, pte_linear); \
+    pte = *pte_p; \
+} while(0)
+#else
+#define PTE_OFFSET(pte, pmd_p, pte_linear) \
+do { \
+    pte_t* pte_p; \
+    pte_p = pte_offset(pmd_p, pte_linear); \
+    pte = *pte_p; \
+} while(0)
+#endif
+#endif
+#endif
+
+#define PTE_PRESENT(pte) \
+do { \
+    if (!pte_present(pte)) \
+    { \
+        __KE_ERROR("FATAL ERROR: User queue buffer not present! (pte)\n"); \
+        return (unsigned long)NOPAGE_SIGBUS;   /* Something bad happened; generate SIGBUS */ \
+        /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
+    } \
+} while(0)
+
+#ifdef __x86_64__
+#define HAS_PTE_HIGH_ADDR(pte) 0
+#else
+#define HAS_PTE_HIGH_ADDR(pte) ((((u64)pte_val(pte) >> 32) & 0xf) ? 1:0)
+#endif
+
 /*****************************************************************************/
 
 struct inode;
@@ -70,7 +191,7 @@ typedef unsigned char __ke_u8;
 typedef unsigned short __ke_u16;
 typedef unsigned int __ke_u32;
 typedef unsigned long long __ke_u64;
-typedef unsigned long __ke_dma_addr_t;
+typedef unsigned long long __ke_dma_addr_t;
 
 // note: assigning uniqe types to originally non interchangeable types
 typedef struct { int uniqe1; } __ke_wait_queue_head_t;
@@ -151,6 +272,7 @@ extern void ATI_API_CALL __ke_remove_wait_queue(__ke_wait_queue_head_t* queue_he
 
 extern void ATI_API_CALL __ke_schedule(void);
 extern int ATI_API_CALL __ke_signal_pending(void);
+extern void ATI_API_CALL __ke_yield(void);
 
 extern void ATI_API_CALL __ke_set_current_state_task_interruptible(void);
 extern void ATI_API_CALL __ke_set_current_state_task_running(void);
@@ -161,6 +283,7 @@ extern void ATI_API_CALL __ke_unblock_all_signals(void);
 extern unsigned long ATI_API_CALL __ke__cmpxchg(volatile void *ptr, unsigned long old,                      
                       unsigned long new, int size);
 extern unsigned long ATI_API_CALL __ke_cpu_to_le32(unsigned long _u);
+extern unsigned long long ATI_API_CALL __ke_cpu_to_le64(unsigned long long _u);
 
 #define __ke_cmpxchg(ptr,o,n)                        \
   ((__typeof__(*(ptr)))__ke__cmpxchg((ptr),(unsigned long)(o),      \
@@ -281,8 +404,9 @@ extern int ATI_API_CALL __ke_verify_read_access(void* addr, __ke_size_t size);
 extern int ATI_API_CALL __ke_verify_write_access(void* addr, __ke_size_t size);
 extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_init_mm(void);
 extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_current_mm(void);
-extern unsigned long ATI_API_CALL __ke_get_vm_phys_addr(struct mm_struct* mm, unsigned long virtual_addr);
-extern unsigned long* ATI_API_CALL __ke_get_vm_phys_addr_list(struct mm_struct* mm, unsigned long virtual_addr, unsigned long pages);
+extern __ke_dma_addr_t ATI_API_CALL __ke_get_vm_phys_addr(struct mm_struct* mm, unsigned long virtual_addr);
+extern unsigned long ATI_API_CALL __ke_get_vm_page_table(struct mm_struct* mm, unsigned long virtual_addr);
+extern __ke_dma_addr_t* ATI_API_CALL __ke_get_vm_phys_addr_list(struct mm_struct* mm, unsigned long virtual_addr, unsigned long pages);
 extern void* ATI_API_CALL __ke_memset(void* s, int c, __ke_size_t count);
 extern void* ATI_API_CALL __ke_memcpy(void* d, const void* s, __ke_size_t count);
 extern ATI_API_CALL __ke_size_t __ke_strlen(const char *s);
@@ -291,6 +415,9 @@ extern char* ATI_API_CALL __ke_strncpy(char* d, const char* s, __ke_size_t count
 extern int ATI_API_CALL __ke_strcmp(const char *s1, const char *s2);
 extern int ATI_API_CALL __ke_strncmp(const char* str1, const char* str2, __ke_size_t count);
 extern int ATI_API_CALL __ke_sprintf(char* buf, const char* fmt, ...);
+extern int ATI_API_CALL __ke_vm_test_and_clear_dirty(struct mm_struct* mm, unsigned long virtual_addr);
+extern void* ATI_API_CALL __ke_vmap(unsigned long *pagelist, unsigned int count);
+extern void ATI_API_CALL __ke_vunmap(void* addr);
 
 /*****************************************************************************/
 
@@ -306,6 +433,7 @@ extern int ATI_API_CALL __ke_flush_cache(void);
 extern int ATI_API_CALL __ke_config_mtrr(void);
 extern int ATI_API_CALL __ke_mtrr_add_wc(unsigned long base, unsigned long size);
 extern int ATI_API_CALL __ke_mtrr_del(int reg, unsigned long base, unsigned long size);
+extern int ATI_API_CALL __ke_has_vmap(void);
 
 #ifdef __x86_64__
 extern int ATI_API_CALL __ke_config_iommu(void);
@@ -357,13 +485,14 @@ enum __ke_vm_maptype
     __KE_AGP_BQS,
     __KE_AGP,
     __KE_SG,
+    __KE_KMAP,
 };
 extern char* ATI_API_CALL __ke_vm_flags_str(struct vm_area_struct* vma, char* buf);
 extern char* ATI_API_CALL __ke_vm_page_prot_str(struct vm_area_struct* vma, char* buf);
 extern char* ATI_API_CALL __ke_vm_phys_addr_str(struct vm_area_struct* vma, 
                                    char* buf, 
                                    unsigned long linear_address, 
-                                   unsigned long* phys_address);
+                                   __ke_dma_addr_t* phys_address);
 extern int ATI_API_CALL __ke_vm_map(struct file* filp,
                        struct vm_area_struct* vma,
                        enum __ke_vm_maptype type,
@@ -404,7 +533,7 @@ typedef struct __ke_agp_kern_info
 
 extern int __ke_agp_try_unsupported;
 
-int ATI_API_CALL __ke_agp_available(int use_internal);
+int ATI_API_CALL __ke_agp_available(__ke_pci_dev_t *pcidev, int use_internal);
 void ATI_API_CALL __ke_agp_uninit(void);
 #ifdef FGL
 struct _agp_memory* ATI_API_CALL __ke_agp_allocate_memory_phys_list(
@@ -417,7 +546,7 @@ int ATI_API_CALL __ke_agp_bind_memory(struct _agp_memory* handle, __ke_off_t sta
 int ATI_API_CALL __ke_agp_unbind_memory(struct _agp_memory* handle);
 int ATI_API_CALL __ke_agp_enable(unsigned long mode);
 int ATI_API_CALL __ke_read_agp_caps_registers(__ke_pci_dev_t* dev, unsigned int *caps);
-int ATI_API_CALL __ke_agp_acquire(void);
+int ATI_API_CALL __ke_agp_acquire(__ke_pci_dev_t* dev);
 void ATI_API_CALL __ke_agp_release(void);
 void ATI_API_CALL __ke_agp_copy_info(__ke_agp_kern_info_t* info);
 unsigned long ATI_API_CALL __ke_agp_memory_handle(struct _agp_memory* handle);
@@ -447,6 +576,19 @@ void ATI_API_CALL firegl_agp_cleanup(void);
 unsigned long ATI_API_CALL firegl_agp_memory_handle(struct _agp_memory* handle);
 unsigned long ATI_API_CALL firegl_agp_memory_page_count(struct _agp_memory* handle);
 
+/* IGP -- related API calls */
+int ATI_API_CALL __ke_agp_memory_get_page_count(struct _agp_memory* agpmem);
+void ATI_API_CALL __ke_agp_memory_get_memory(struct _agp_memory* agpmem,
+                                unsigned long **memory_ptr);
+unsigned long ATI_API_CALL firegl_igp_get_agpmem_phys_addr(
+                                                void*         drm_filp,
+                                                unsigned long virtual_addr,
+                                                unsigned long vm_start,
+                                                unsigned long vm_end,
+                                                unsigned long handle);
+int ATI_API_CALL firegl_is_igp_rs400(void *priv);
+int ATI_API_CALL firegl_is_igp_rs480(void *priv);
+
 #ifdef __x86_64__
 int ATI_API_CALL firegl_call_ioctl(unsigned int fd, unsigned int cmd, void *data);
 int ATI_API_CALL __ke_sys_ioctl(unsigned int fd, unsigned int cmd, void *data);
@@ -466,6 +608,8 @@ int ATI_API_CALL firegl_put_user_u64(__ke_u64 src,   __ke_u32 *dst);
 
 extern __ke_dma_addr_t ATI_API_CALL __ke_pci_map_single (__ke_pci_dev_t *pdev, void *buffer, __ke_size_t size, int direction); 
 extern void ATI_API_CALL __ke_pci_unmap_single (__ke_pci_dev_t *pdev, __ke_dma_addr_t bus_addr, __ke_size_t size, int direction); 
+extern __ke_dma_addr_t ATI_API_CALL __ke_pci_map_page (__ke_pci_dev_t *pdev, unsigned long buffer, unsigned long offset, __ke_size_t size, int direction);
+extern void ATI_API_CALL __ke_pci_unmap_page (__ke_pci_dev_t *pdev, __ke_dma_addr_t bus_addr, __ke_size_t size, int direction); 
 extern int ATI_API_CALL __ke_log2(unsigned long x);
 
 
@@ -501,6 +645,7 @@ extern const unsigned long  __ke_PAGE_MASK;
 
 extern const unsigned long  __ke_MODVERSIONS_State;
 extern const unsigned long  __ke_SMP_State;
+extern const unsigned long  __ke_PAE_State;
 
 /* global vars that are in fact constants */
 extern unsigned long        __ke_HZ;
