@@ -1,6 +1,6 @@
 /*-
- * Copyright (c) 2002-2004 Sam Leffler, Errno Consulting
- * Copyright (c) 2004 Atheros Communications, Inc.
+ * Copyright (c) 2002-2005 Sam Leffler, Errno Consulting
+ * Copyright (c) 2004-2005 Atheros Communications, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: if_ath_pci.h,v 1.2 2004/12/29 05:38:07 samleffler Exp $
+ * $Id: if_ath_pci.h,v 1.4 2005/07/11 13:23:54 br1 Exp $
  */
  
 #ifndef _DEV_ATH_PCI_H_
@@ -45,10 +45,13 @@
 #define bus_unmap_single	pci_unmap_single
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 #define bus_dma_sync_single	pci_dma_sync_single_for_cpu
+#define BUS_DMA_MAP_ERROR(a)	pci_dma_mapping_error(a)
 #define	PCI_SAVE_STATE(a,b)	pci_save_state(a)
 #define	PCI_RESTORE_STATE(a,b)	pci_restore_state(a)
 #else
 #define bus_dma_sync_single	pci_dma_sync_single
+/* AFAIK we can't reliably check for DMA mapping errors on 2.4 kernels */
+#define BUS_DMA_MAP_ERROR(a)	0
 #define	PCI_SAVE_STATE(a,b)	pci_save_state(a,b)
 #define	PCI_RESTORE_STATE(a,b)	pci_restore_state(a,b)
 #endif
