@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2002-2004 Sam Leffler, Errno Consulting, Atheros
+ * Copyright (c) 2002-2005 Sam Leffler, Errno Consulting, Atheros
  * Communications, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -33,7 +33,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGES.
  *
- * $Id: ah_osdep.h,v 1.1.1.9 2004/10/20 18:16:08 samleffler Exp $
+ * $Id: ah_osdep.h,v 1.4 2005/02/23 17:25:20 otaku Exp $
  */
 #ifndef _ATH_AH_OSDEP_H_
 #define _ATH_AH_OSDEP_H_
@@ -53,6 +53,9 @@
 #else
 #define	__ahdecl
 #endif
+#ifndef __packed
+#define	__packed	__attribute__((__packed__))
+#endif
 
 /*
  * When building the HAL proper we use no GPL-contaminated include
@@ -150,6 +153,22 @@ __bswap32(u_int32_t _x)
  * register values are constants.
  */
 #if AH_BYTE_ORDER == AH_BIG_ENDIAN
+
+#if ( defined(CONFIG_PPC_PMAC) || defined(CONFIG_ARCH_IXP425) || defined(CONFIG_ARCH_IXP4XX)) /* ixp4xx or PowerPC architecture */
+
+#define _OS_REG_WRITE(_ah, _reg, _val) do {				    \
+	if ( (_reg) >= 0x4000 && (_reg) < 0x5000)			    \
+	    writel((_val), (volatile unsigned long)((_ah)->ah_sh + (_reg)));\
+	else								    \
+	    writel(__bswap32(_val), (volatile unsigned long)((_ah)->ah_sh + (_reg))); \
+} while (0)
+#define _OS_REG_READ(_ah, _reg) \
+	(((_reg) >= 0x4000 && (_reg) < 0x5000) ?			    \
+	    readl((volatile unsigned long)((_ah)->ah_sh + (_reg))) :	    \
+	    __bswap32(readl((volatile unsigned long)((_ah)->ah_sh + (_reg)))))
+
+#else  /* normal case */
+
 #define _OS_REG_WRITE(_ah, _reg, _val) do {				    \
 	if ( (_reg) >= 0x4000 && (_reg) < 0x5000)			    \
 		*((volatile u_int32_t *)((_ah)->ah_sh + (_reg))) =	    \
@@ -161,6 +180,8 @@ __bswap32(u_int32_t _x)
 	(((_reg) >= 0x4000 && (_reg) < 0x5000) ? \
 		__bswap32(*((volatile u_int32_t *)((_ah)->ah_sh + (_reg)))) : \
 		*((volatile u_int32_t *)((_ah)->ah_sh + (_reg))))
+
+#endif
 #else /* AH_LITTLE_ENDIAN */
 #define _OS_REG_WRITE(_ah, _reg, _val) do { \
 	*((volatile u_int32_t *)((_ah)->ah_sh + (_reg))) = (_val); \
