@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2004 Sam Leffler, Errno Consulting
+ * Copyright (c) 2004-2005 Sam Leffler, Errno Consulting
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -702,7 +702,7 @@ init_crypto_ccmp_test(void)
 
 	memset(&ic, 0, sizeof(ic));
 	if (debug)
-		ic.msg_enable = IEEE80211_MSG_CRYPTO;
+		ic.ic_debug = IEEE80211_MSG_CRYPTO;
 	ieee80211_crypto_attach(&ic);
 
 	pass = 0;
