@@ -5,8 +5,7 @@
 # source tree so it can be built statically.  Typically this
 # is done to simplify debugging with tools like kgdb.
 #
-KERNEL_VERSION=`uname -r | \
-		awk '{ split($0,a,"."); print a[1] "." a[2]; }'`
+KERNEL_VERSION=`uname -r`
 KERNEL_PATH=${1:-/lib/modules/${KERNEL_VERSION}/build}
 
 DEPTH=../..
@@ -31,7 +30,7 @@ INSTALL()
 INSTALLX()
 {
 	DEST=$1; shift
-	sed -e '/^##2.4##/d' -e 's/^##2.6##//' $1 > $DEST
+	sed -e 's/^##2.4##//' -e '/^##2.6##/d' $1 > $DEST
 }
 
 #
@@ -92,16 +91,16 @@ MKDIR ${DST_HAL}/linux
 INSTALL ${DST_HAL}/linux ${SRC_HAL}/linux/ah_osdep.c
 INSTALL ${DST_HAL}/linux ${SRC_HAL}/linux/ah_osdep.h
 # XXX copy only target or use arch? 
-INSTALL ${DST_HAL}/linux ${SRC_HAL}/linux/*.inc
-INSTALL ${DST_HAL}/linux ${SRC_HAL}/linux/*.opt_ah.h
-INSTALL ${DST_HAL}/linux ${SRC_HAL}/linux/*.hal.o.uu
+MKDIR ${DST_HAL}/public
+INSTALL ${DST_HAL}/public ${SRC_HAL}/public/*.inc
+INSTALL ${DST_HAL}/public ${SRC_HAL}/public/*.opt_ah.h
+INSTALL ${DST_HAL}/public ${SRC_HAL}/public/*.hal.o.uu
 if [ -d ${SRC_HAL}/ar5212 ]; then
 	MKDIR ${DST_HAL}/ar5212
 	INSTALL ${DST_HAL}/ar5212 ${SRC_HAL}/ar5212/ar5212desc.h
 fi
 
 DST_NET80211=${WIRELESS}/net80211
-DST_DOC=${KERNEL_PATH}/Documentation
 MKDIR ${DST_NET80211}
 echo "Copy net80211 bits..."
 FILES=`ls ${SRC_NET80211}/*.[ch] | sed '/mod.c/d'`
@@ -115,7 +114,10 @@ INSTALL ${DST_NET80211}/compat ${SRC_COMPAT}/compat.h
 MKDIR ${DST_NET80211}/compat/sys
 INSTALL ${DST_NET80211}/compat/sys ${SRC_COMPAT}/sys/*.h
 
-grep -q 'ath/Kconfig' ${WIRELESS}/Kconfig || \
-	PATCH ${WIRELESS}/Kconfig Kconfig.patch
-grep -q '_ath_hal' ${WIRELESS}/Makefile || \
+grep -q 'CONFIG_ATHEROS' ${WIRELESS}/Config.in || \
+	PATCH ${WIRELESS}/Config.in Config.in.patch
+grep -q 'CONFIG_ATHEROS' ${WIRELESS}/Makefile || \
 	PATCH ${WIRELESS}/Makefile Makefile.patch
+DST_DOC=${KERNEL_PATH}/Documentation
+grep -q 'CONFIG_ATHEROS' ${DST_DOC}/Configure.help || \
+	PATCH ${DST_DOC}/Configure.help Configure.help.patch
